from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting

from plone.testing import z2

from zope.configuration import xmlconfig


class ItyouqrcodeLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import ityou.qrcode
        xmlconfig.file(
            'configure.zcml',
            ityou.qrcode,
            context=configurationContext
        )

        # Install products that use an old-style initialize() function
        #z2.installProduct(app, 'Products.PloneFormGen')

#    def tearDownZope(self, app):
#        # Uninstall products installed above
#        z2.uninstallProduct(app, 'Products.PloneFormGen')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ityou.qrcode:default')

ITYOU_QRCODE_FIXTURE = ItyouqrcodeLayer()
ITYOU_QRCODE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(ITYOU_QRCODE_FIXTURE,),
    name="ItyouqrcodeLayer:Integration"
)
ITYOU_QRCODE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(ITYOU_QRCODE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="ItyouqrcodeLayer:Functional"
)



from zope.interface import Interface
# -*- Additional Imports Here -*-


class IItyouQrcodeLayer(Interface):
    """ A layer specific to this product.
        this layer is registered using browserlayer.xml in the package 
        default GenericSetup profile
    """


# -*- coding: utf-8 -*-
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName 
from Acquisition import aq_inner

from .PyQRNative import QRCode, QRErrorCorrectLevel
from . import INSTANCE_PATH, QRCODE_LOCATION, QRCODE_RESOURCE

class QRCodeUtils:
    
    @memoize
    def create_qrcode(self,context):
        """ Creates and returns qr code
        """
        url = context.absolute_url()
        uid = context.UID()

        if len(url) < 78:
            type_number = 4
        elif len(url) < 107:
            type_number = 5
        elif len(url) < 135:
            type_number = 7
        else:
            type_number = 8

        qr = QRCode(type_number, QRErrorCorrectLevel.L)
        qr.addData(url)
        qr.make()

        im = qr.makeImage()
        im.save( '%s/%s.png' % (QRCODE_LOCATION,uid) )
        ###im.show()

        return '<img src="%s/%s.png" alt="%s" title="%s" class="image-inline img-responsive" />' % (
            QRCODE_RESOURCE,
            uid,
            context.title_or_id(),
            context.title_or_id()
            ) 
    
    def qrcode_url(self,context):
        """returns the url of the QR Code
        """
        url = context.absolute_url()
        uid = context.UID()

        url = url.replace('/view','') + '/%s/%s.png' % (QRCODE_RESOURCE, uid)
        print url
        return url

        



# -*- coding: utf-8 -*-
"""Define a portlet used to show grouplist. This follows the patterns from
plone.app.portlets.portlets. Note that we also need a portlet.xml in the
GenericSetup extension profile to tell Plone about our new portlet.
"""

import random

from Acquisition import aq_inner

from zope import schema
from zope.formlib import form
from zope.interface import implements
from zope.component import getMultiAdapter

from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider

from DateTime import DateTime
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.CMFCore.utils import getToolByName
from ..utils import QRCodeUtils
from .. import _

##from .utils import WhoAmIUtils

class IQRCodePortlet(IPortletDataProvider):

    show_all = schema.Bool(
            title=_(u"Display QR Code"),
            description=_(u"Show the genetated QR Code"),
            required=True,
            default=True,
        )

class Assignment(base.Assignment):
    implements(IQRCodePortlet)

    def __init__(self, show_all=True):
        self.show_all = show_all

    title = _(u"QRCode")

class Renderer(base.Renderer):

    _template = ViewPageTemplateFile('qrcode.pt')
    
    def render(self):

        context = aq_inner(self.context)
        mt = getToolByName(context, 'portal_membership')
        
        #if not mt.isAnonymousUser():
        return self._template()
               
    @property
    def available(self):
        return True

    def view_qrcode(self):
        """Create a new qrcode of the page
        """
        context = aq_inner(self.context)

        qru = QRCodeUtils()
        return qru.create_qrcode(context)

    def qrcode_url(self):
        """Returns URL of QR Code
        """
        context = aq_inner(self.context)

        qru = QRCodeUtils()
        return qru.qrcode_url(context)


class AddForm(base.AddForm):
    form_fields = form.Fields(IQRCodePortlet)
    label = _(u"Add QRCode portlet")
    description = _(u"This portlet displays online users.")

    def create(self, data):
        assignment = Assignment()
        form.applyChanges(assignment, self.form_fields, data)
        return assignment

class EditForm(base.EditForm):
    form_fields = form.Fields(IQRCodePortlet)
    label = _(u"Edit QRCode portlet")
    description = _(u"This portlet displays online users.")
    



# -*- extra stuff goes here -*-
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('ityou.qrcode')


INSTANCE_PATH   = '/'.join(INSTANCE_HOME.split('/')[:-2])
QRCODE_LOCATION = INSTANCE_PATH + "/src/ityou.qrcode/src/ityou/qrcode/qr_codes"
QRCODE_RESOURCE = "++resource++ityou.qrcodes"

BOXSIZE = 10 # Pixel
OFFSET  = 4 # Boxes
BOXCOLOR = '#596069' # grau


def initialize(context):
    """Initializer called when used as a Zope 2 product."""
